#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.features.header.interfaces import IDefaultHeaderRendererSettings, IDefaultHeaderMenu, \
    IDefaultHeaderMenusContainer
from pyams_content.component.association.interfaces import ASSOCIATION_CONTAINER_KEY
from pyams_content.features.menu.interfaces import IMenusContainer, IMenusContainerTarget, \
    IMenuLinksContainerTarget, IMenuLinksContainer
from pyams_content.component.illustration.interfaces import IIllustration
from pyams_content.features.header.interfaces import IHeaderTarget
from zope.location.interfaces import ISublocations

# import packages
from persistent import Persistent
from pyams_content.features.menu import MenusContainer, Menu
from pyams_file.property import FileProperty
from pyams_utils.adapter import adapter_config, get_annotation_adapter, ContextAdapter
from pyams_utils.traversing import get_parent
from zope.interface import implementer
from zope.location import Location, locate
from zope.schema.fieldproperty import FieldProperty


#
# Default header renderer settings
#

DEFAULT_HEADER_TOP_LINKS_KEY = '{0}::top_links'.format(ASSOCIATION_CONTAINER_KEY)
DEFAULT_HEADER_MENUS_KEY = '{0}::menus'.format(ASSOCIATION_CONTAINER_KEY)


@implementer(IDefaultHeaderRendererSettings, IMenuLinksContainerTarget, IMenusContainerTarget)
class DefaultHeaderRendererSettings(Persistent, Location):
    """Default header renderer settings"""

    _use_context_illustration = FieldProperty(IDefaultHeaderRendererSettings['use_context_illustration'])
    background = FileProperty(IDefaultHeaderRendererSettings['background'])
    heading = FieldProperty(IDefaultHeaderRendererSettings['heading'])
    logo = FileProperty(IDefaultHeaderRendererSettings['logo'])
    scroll_logo = FileProperty(IDefaultHeaderRendererSettings['scroll_logo'])
    accessibility_menu = FieldProperty(IDefaultHeaderRendererSettings['accessibility_menu'])
    search_menu = FieldProperty(IDefaultHeaderRendererSettings['search_menu'])

    @property
    def can_use_context_illustration(self):
        target = get_parent(self, IHeaderTarget)
        illustration = IIllustration(target, None)
        return (illustration is not None) and bool(illustration.data)

    @property
    def use_context_illustration(self):
        return self._use_context_illustration if self.can_use_context_illustration else False

    @use_context_illustration.setter
    def use_context_illustration(self, value):
        if self.can_use_context_illustration:
            self._use_context_illustration = value

    @property
    def no_use_context_illustration(self):
        return not bool(self.use_context_illustration)

    @no_use_context_illustration.setter
    def no_use_context_illustration(self, value):
        self.use_context_illustration = not bool(value)

    @property
    def top(self):
        return get_annotation_adapter(self, DEFAULT_HEADER_TOP_LINKS_KEY, Menu,
                                      markers=IDefaultHeaderMenu, name='++ass++top')

    @property
    def menus(self):
        return get_annotation_adapter(self, DEFAULT_HEADER_MENUS_KEY, MenusContainer,
                                      markers=IDefaultHeaderMenusContainer, name='++ass++menus')


@adapter_config(name='top', context=IDefaultHeaderRendererSettings, provides=IMenuLinksContainer)
def default_header_top_links_adapter(context):
    """Default header top links factory"""
    return context.top


@adapter_config(name='menus', context=IDefaultHeaderRendererSettings, provides=IMenusContainer)
def default_header_menus_factory(context):
    """Default header middle links factory"""
    return context.menus


@adapter_config(name='links', context=IDefaultHeaderRendererSettings, provides=ISublocations)
class DefaultHeaderRendererSettingsSublocations(ContextAdapter):
    """Default header renderer settings sub-locations adapter"""

    def sublocations(self):
        return self.context.top.values() + self.context.menus.values()
